#include "bitmaps.h"
#include <WiFi.h>;
#include <HTTPClient.h>;
#include <Arduino_JSON.h>;
#include <TFT_eSPI.h> 
#include <SPI.h>
#include <Bme280.h>

#define SW2 18

const char* ssid = "xxxxxxx";
const char* password = "xxxx";
String my_Api_Key = "xxxxxxxxxxxxxx";

String my_city = "Atlanta"; //specify your city
String my_country_code = "US"; //specify your country code

String json_array;

Bme280TwoWire sensor;

TFT_eSPI tft = TFT_eSPI();  //pins defined in User_Setup.h

void setup() {
  pinMode(SW2, INPUT_PULLDOWN);
  Serial.begin(9600);
  Wire.begin();
  sensor.begin(Bme280TwoWireAddress::Primary);
  sensor.setSettings(Bme280Settings::indoor());
  tft.init();
  tft.setRotation(3);
}

void loop() {
  show_wifi_setup();
  delay(3000);
  while (digitalRead(SW2) == 0);  //wait for SW2 press
  while(1) {
    show_BME280_measurements();
    delay(500);
    while (digitalRead(SW2) == 0);  //wait for SW2 press
    show_OpenWeather();
    delay(500);
    while (digitalRead(SW2) == 0);  //wait for SW2 press
    // OPTION: show_WindRain();
    // OPTION: show_Lightning();
  }
}

void show_wifi_setup(){
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("Connecting to Wi-Fi...");
  tft.drawString("Connecting to Wi-Fi...", 20, 40, 4);
  while (WiFi.status() != WL_CONNECTED) {
    Serial.println("Wi-Fi Not Connected.");
    delay(1000);
  }
  Serial.println("");
  Serial.println("Wi-Fi Connected.");
  Serial.print("IP:");
  Serial.println(WiFi.localIP());
  tft.drawString("Wi-Fi Connected.", 20, 100, 4);
  tft.drawString("Press SW2 to", 20, 165, 4);
  tft.drawString("    Advance Display.", 20, 190, 4);
}

void show_BME280_measurements(){
  tft.fillScreen(TFT_BLACK);
  tft.drawXBitmap(27, 50, temp, iconWidth, iconHeight, TFT_RED);
  tft.drawXBitmap(27, 105, humidity, iconWidth, iconHeight, TFT_BLUE);
  tft.drawXBitmap(27, 160, pressure, iconWidth, iconHeight, TFT_YELLOW);
  
  float tempC = sensor.getTemperature();
  auto temperatureC = String(tempC) + " C";
  float tempF = ((tempC * 9) / 5) + 32;
  auto temperatureF = String(tempF) + " F";
  auto pressure = String(sensor.getPressure() / 100.0) + " hPa";
  auto humidity = String(sensor.getHumidity()) + " %";

  tft.setTextColor(TFT_WHITE);
  tft.drawString("Indoor BME280 Readings", 20, 10, 4);
  tft.setTextColor(TFT_RED);
  String measurements = temperatureC + " / " + temperatureF;
  tft.drawString(measurements, 95, 65, 4);
  tft.setTextColor(TFT_BLUE);
  tft.drawString(humidity, 95, 120, 4);
  tft.setTextColor(TFT_YELLOW);
  tft.drawString(pressure, 95, 175, 4);
}

void show_OpenWeather(){
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE);
  tft.drawString("Outdoor OpenWeather", 20, 7, 4);
  if (WiFi.status() == WL_CONNECTED) {
    String server = "http://api.openweathermap.org/data/2.5/weather?q=" + my_city + "," + my_country_code + "&APPID=" + my_Api_Key + "&units=imperial";
    json_array = GET_Request(server.c_str());
    Serial.println(json_array);
    JSONVar my_obj = JSON.parse(json_array);
    if (JSON.typeof(my_obj) == "undefined") {
      Serial.println("Parsing input failed!");
      return;
    }
    Serial.println("JSON object = ");
    Serial.println(my_obj);
    Serial.print("Temperature: ");
    Serial.println(my_obj["main"]["temp"]);
    Serial.print("Humidity: ");
    Serial.println(my_obj["main"]["humidity"]);
    Serial.print("Pressure: ");
    Serial.println(my_obj["main"]["pressure"]);
    Serial.print("Wind Speed: ");
    Serial.println(my_obj["wind"]["speed"]);

    tft.drawXBitmap(27, 42, temp, iconWidth, iconHeight, TFT_RED);
    tft.drawXBitmap(27, 95, humidity, iconWidth, iconHeight, TFT_BLUE);
    tft.drawXBitmap(27, 147, pressure, iconWidth, iconHeight, TFT_YELLOW);
    tft.drawXBitmap(27, 199, wind, iconWidth, iconHeight, TFT_GREEN);
        
    tft.setTextColor(TFT_RED);
    String measurement = JSON.stringify(my_obj["main"]["temp"]) + " F";
    tft.drawString(measurement, 95, 57, 4);
    tft.setTextColor(TFT_BLUE);
    measurement = JSON.stringify(my_obj["main"]["humidity"]) + " %";
    tft.drawString(measurement, 95, 110, 4);
    tft.setTextColor(TFT_YELLOW);
    measurement = JSON.stringify(my_obj["main"]["pressure"]) + " hPa";
    tft.drawString(measurement, 95, 162, 4);
    tft.setTextColor(TFT_GREEN);
    measurement = JSON.stringify(my_obj["wind"]["speed"]) + " mph";
    tft.drawString(measurement, 95, 214, 4);
  }
  else {
    Serial.println("WiFi Disconnected");
    tft.drawString("WiFi Disconnected", 20, 110, 4);
  }
}

String GET_Request(const char* server) {
  HTTPClient http;
  http.begin(server);
  int httpResponseCode = http.GET();
  String payload = "{}";
  if (httpResponseCode > 0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  }
  else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  http.end();
  return payload;
}
